@@ -1,17 +1,14 @@
-;; @(#) czech-convert.el -- conversions between various Czech encodings
-
-;; @(#) $Id: czech-convert.el,v 3.4 1997/02/26 21:12:49 pdm Exp $
-;; @(#) $Keywords: i18n, Czech, encoding, conversion $
-;; $KnownCompatibility: 19.34, XEmacs 19.14 $
-
-;; This file is *NOT* part of GNU Emacs nor XEmacs.
+;;; czech-convert.el -- conversions between various Czech encodings
 
 ;; Copyright (C) 1995, 1996, 1997 Milan Zamazal
 
-;; Author:       Milan Zamazal <pdm@fi.muni.cz>
-;; Maintainer:   Milan Zamazal <pdm@fi.muni.cz>
-;; Requires:     czech.el
-;; Remark:       Don't laugh too loudly while reading this file, please.
+;; Author:     Milan Zamazal <pdm@fi.muni.cz>
+;; Maintainer: Milan Zamazal <pdm@fi.muni.cz>
+;; Version:    $Id: czech-convert.el,v 3.6 1997/06/25 15:22:05 pdm Exp $
+;; Keywords:   i18n, Czech, encodings, conversion
+;; KnownCompatibility: 19.34, XEmacs 19.14, 19.15
+
+;; This file is *NOT* part of GNU Emacs nor XEmacs.
 
 ;; COPYRIGHT NOTICE
 ;;
@@ -45,7 +42,7 @@
 
 (require 'czech)
 
-(defconst cz-convert-version "$Id: czech-convert.el,v 3.4 1997/02/26 21:12:49 pdm Exp $"
+(defconst cz-convert-version "$Id: czech-convert.el,v 3.6 1997/06/25 15:22:05 pdm Exp $"
   "Latest modification time and version number.")
 
 (defun cz-convert-message-start (begin end)
@@ -147,7 +144,7 @@ appropriate files."
 ;;;###autoload
 (defun cz-convert-to-csech (begin end &optional append-message)
   "Converts region from ISO 8859-2 to 7-bit ASCII (so called \"csech\").
-If APPEND-MESSAGE is non-`nil' or prefix argument is given, it appends message
+If APPEND-MESSAGE is non-nil or prefix argument is given, it appends message
 about conversion to end of the converted region.  It converts all 8-bit
 characters, but those that are not diacritic will be converted to dots."
   (interactive "*r")
@@ -171,7 +168,7 @@ esk text konvertovn na cesky text.
   "Converts region from brothers' Kamenit encoding to ISO 8859-2.
 Brothers' Kamenit encoding was the most widenly used encoding before
 Micro$oft attack.
-If an optional argument LEAVE-CTRL-M is non-`nil', do not remove ^M's."
+If an optional argument LEAVE-CTRL-M is non-nil, do not remove ^M's."
   (interactive "*r")
   (cz-convert-message-start begin end)
   (let ((output (if cz-windoze 'win 'il2)))
@@ -191,7 +188,7 @@ KOI-8 is Soviet Union encoding used at least at one place in Czech Republic
 yet.  This function does not convert `ch' and `CH' characters properly.  Also
 the characters \\`a and \\`A aren't converted correctly -- ISO 8859-2 does not
 contain them.
-If an optional argument LEAVE-CTRL-M is non-`nil', do not remove ^M's."
+If an optional argument LEAVE-CTRL-M is non-nil, do not remove ^M's."
   (interactive "*r")
   (cz-convert-message-start begin end)
   (let ((output (if cz-windoze 'win 'il2)))
@@ -209,7 +206,7 @@ If an optional argument LEAVE-CTRL-M is non-`nil', do not remove ^M's."
   "Converts region from M$-Windoze (Czech version) encoding to ISO 8859-2.
 Czech M$-Windoze encoding (\"code page 1250\") is some mutation of ISO 8859-2
 used by Micro$oft in their Windoze interface.
-If an optional argument LEAVE-CTRL-M is non-`nil', do not remove ^M's."
+If an optional argument LEAVE-CTRL-M is non-nil, do not remove ^M's."
   (interactive "*r")
   (cz-convert-message-start begin end)
   (if (cz-use-cstocs begin end)
@@ -225,7 +222,7 @@ If an optional argument LEAVE-CTRL-M is non-`nil', do not remove ^M's."
 (defun cz-convert-from-do$ (begin end &optional leave-ctrl-m)
   "Converts region from \"Code page 852\" to ISO 8859-2.
 \"Code page 852\" is very stupid encoding introduced by Micro$oft here.
-If an optional argument LEAVE-CTRL-M is non-`nil', do not remove ^M's."
+If an optional argument LEAVE-CTRL-M is non-nil, do not remove ^M's."
   (interactive "*r")
   (cz-convert-message-start begin end)
   (let ((output (if cz-windoze 'win 'il2)))
@@ -254,6 +251,122 @@ similar functions and has no effect."
 					 cz-convert-internal-tables))))))
   (cz-convert-message-end begin end))
 
+;; Di/trigraphs to ISO
+;;;###autoload
+(defun cz-convert-from-digraphs (beg end)
+  "Convert from digraphs encoding.
+That encoding is used by some people around Masaryk University and is of no
+importance.
+Because there is no standard of this encoding, magical encoding detection is
+performed and appropriate action is chosen."
+  (interactive "*r")
+  (let ((end-marker (copy-marker end))
+	(digraphs-list cz-convert-digraphs)
+	one-digraph-list
+	replace-list
+	regexp
+	aux
+	digraph
+	replacement
+	max)
+    (save-excursion
+      ;; Build replacement list
+      (while digraphs-list
+	(setq one-digraph-list (cdr (car digraphs-list))
+	      digraph (car one-digraph-list)
+	      one-digraph-list (cdr one-digraph-list))
+	;; Simple heuristic: We choose the most often repeating di/trigraphs
+	;; element as the right one used by user.
+	(if (not (null one-digraph-list))
+	    (progn
+	      (setq max (cz-convert-digraphs-count digraph beg end))
+	      (while one-digraph-list
+		(setq aux (cz-convert-digraphs-count
+			   (car one-digraph-list) beg end))
+		(if (> aux max)
+		    (setq digraph (car one-digraph-list)
+			  aux max))
+		(setq one-digraph-list (cdr one-digraph-list)))))
+	(setq replace-list (cons (cons digraph (car (car digraphs-list)))
+				 replace-list)
+	      regexp (if (null regexp) digraph (concat regexp "\\|" digraph)))
+	(setq digraphs-list (cdr digraphs-list)))
+      ;; Replace di/trigraphs
+      (goto-char beg)
+      (while (and (<= (point) (marker-position end-marker))
+		  (re-search-forward regexp end-marker t))
+	(setq digraph (buffer-substring-no-properties (match-beginning 0)
+						      (match-end 0))
+	      replacement (cdr (assoc (downcase digraph) replace-list)))
+	(if (not (eq (aref digraph 0) (downcase (aref digraph 0))))
+	    (setq replacement
+		  (if (eq (aref digraph 1) (downcase (aref digraph 1)))
+		      (capitalize replacement)
+		    (upcase replacement))))
+	(delete-region (match-beginning 0) (match-end 0))
+	(insert replacement))
+      ;; Remove backslashes and pipes
+      (goto-char beg)
+      (while (and (<= (point) (marker-position end-marker))
+		  (re-search-forward "\\\\\\||" end-marker t))
+	(delete-char -1)
+	(goto-char (1+ (point)))))))
+
+(defun cz-convert-digraphs-count (digraph beg end)
+  "Count number of digraphs DIGRAPH in region."
+  (goto-char beg)
+  (let ((n 0))
+    (while (search-forward digraph end t)
+      (setq n (1+ n)))
+    n))
+
+;; ISO to di/trigraphs
+;;;###autoload
+(defun cz-convert-to-digraphs (beg end)
+  "Convert to digraphs encoding.
+That encoding is used by some people around Masaryk University and is of no
+importance."
+  (interactive "*r")
+  (let* ((end-marker (copy-marker end))
+	 (aux (car cz-convert-digraphs))
+	 (aux-list (cdr cz-convert-digraphs))
+	 (regexp (concat (car aux) "\\|" (car (cdr aux)))))
+    (save-excursion
+      ;; Build regular expression
+      (while aux-list
+	(setq aux (car aux-list))
+	(setq regexp (concat regexp "\\|" (car aux) "\\|" (car (cdr aux))))
+	(setq aux-list (cdr aux-list)))
+      ;; Start searching
+      (goto-char beg)
+      (while (and (<= (point) (marker-position end-marker))
+		  (re-search-forward regexp end-marker t))
+	(setq aux-list cz-convert-digraphs
+	      aux (buffer-substring-no-properties
+				   (match-beginning 0)
+				   (match-end 0)))
+	(while (and aux-list
+		    (not (string= (downcase aux) (car (car aux-list)))))
+	  (setq aux-list (cdr aux-list)))
+	(if aux-list
+	    ;; Replace by di/trigraph
+	    (progn
+	      (setq aux (car (cdr (car aux-list))))
+	      (goto-char (match-beginning 0))
+	      (if (not (eq (following-char) (downcase (following-char))))
+		  (setq aux (if (save-excursion
+				  (forward-char)
+				  (skip-chars-forward " \t-")
+				  (eq (following-char)
+				      (downcase (following-char))))
+				(capitalize aux)
+			      (upcase aux))))
+	      (delete-region (match-beginning 0) (match-end 0))
+	      (insert aux))
+	  ;; Insert backslash or pipe
+	  (goto-char (1+ (match-beginning 0)))
+	  (insert cz-convert-digraphs-specchar))))))
+
 
 ;;; *** Magical conversions to ISO 8859-2 ***
 
@@ -336,7 +449,7 @@ Returns symbol of the identified encoding."
 (defun cz-convert-test-region (target-buffer begin end &optional enc-function)
   "Copies region to auxiliary buffer and tests it for given encoding.
 Tests only the part specified by user variables used in `cz-is-cz-text'.
-If ENC-FUNCTION is `nil', no conversion is performed before testing."
+If ENC-FUNCTION is nil, no conversion is performed before testing."
   (save-excursion
     (set-buffer target-buffer)
     (delete-region (point-min) (point-max)))
@@ -479,10 +592,10 @@ where
 - the second element is the regular of all alternatives needing more than one
   character substitution
 - and the third element is substiting string for them.
-If the value of variable `cz-convert-cache-file' is non-`nil', tries firstly
+If the value of variable `cz-convert-cache-file' is non-nil, tries firstly
 to load data from the file specified by that variable.  Also new data are
 appended to this file.
-If NOMESSAGE is non-`nil', no messages are displayed during operation."
+If NOMESSAGE is non-nil, no messages are displayed during operation."
   (let ((conversion-name
 	 (intern
 	  (concat "cz-conv-" input-encoding "-" output-encoding)))
@@ -594,7 +707,7 @@ File is searched in the following order:
 If the file is found, it is read into buffer `cz-convert-aux-buffer' -- its
 previous contents is deleted and point is set to its beginning.
 If the file is not found, error is forced.  If an optional argument NO-ERROR is
-non-`nil', no error is forced and `nil' is returned."
+non-nil, no error is forced and nil is returned."
   (cond
    ((file-readable-p file) file)
    ((file-readable-p (concat file ".enc")) (concat file ".enc"))
@@ -901,7 +1014,7 @@ KOI-8, M$-Windoze (\"CP 1250\"), and DO$ (\"CP 852\")."
 ;;;###autoload
 (defun cz-convert-auto-set (encoding)
   "Noninteractive variation of `cz-convert-auto-encoding'.
-If ENCODING is `nil', autoconversion is disabled for the buffer."
+If ENCODING is nil, autoconversion is disabled for the buffer."
   ;; Initialize autoconversions
   (if (not cz-convert-auto-flag)
       (progn
