@@ -1,17 +1,14 @@
-;; @(#) czech-misc.el -- miscellaneous functions
-
-;; @(#) $Id: czech-misc.el,v 3.4 1997/02/26 20:46:34 pdm Exp $	
-;; @(#) $Keywords: i18n, Czech, fonts, autodetection $
-;; $KnownCompatibility: 19.34, XEmacs 19.14$
-
-;; This file is *NOT* part of GNU Emacs nor XEmacs.
+;;; czech-misc.el -- miscellaneous functions
 
 ;; Copyright (C) 1995, 1996, 1997 Milan Zamazal
 
-;; Author:       Milan Zamazal <pdm@fi.muni.cz>
-;; Maintainer:   Milan Zamazal <pdm@fi.muni.cz>
-;; Requires:     czech.el
-;; Remark:       Don't laugh too loudly while reading this file, please.
+;; Author:     Milan Zamazal <pdm@fi.muni.cz>
+;; Maintainer: Milan Zamazal <pdm@fi.muni.cz>
+;; Version:    $Id: czech-misc.el,v 3.6 1997/06/25 15:05:07 pdm Exp $
+;; Keywords:   i18n, Czech, fonts, autodetection
+;; KnownCompatibility: 19.34, XEmacs 19.14, 19.15
+
+;; This file is *NOT* part of GNU Emacs.
 
 ;; COPYRIGHT NOTICE
 ;;
@@ -49,7 +46,7 @@
 
 (require 'czech)
 
-(defconst cz-misc-version "$Id: czech-misc.el,v 3.4 1997/02/26 20:46:34 pdm Exp $"
+(defconst cz-misc-version "$Id: czech-misc.el,v 3.6 1997/06/25 15:05:07 pdm Exp $"
   "Latest modification time and version number.")
 
 
@@ -323,10 +320,10 @@ If an optional prefix argument is given, last selected character is inserted."
 (defun cz-is-cz-text (&optional return-ratio test-all)
   "Tries to recognize, whether current buffer contains ISO-8859-2 text.
 If the buffer is narrowed, only the narrowed part is considered.
-If an optional argument RETURN-RATIO is non-`nil', list
+If an optional argument RETURN-RATIO is non-nil, list
 `(ratio . ratio-nonczech)' is returned (floating point numbers)
 instead of boolean value.
-If TEST-ALL is non-`nil', test region without considering variables
+If TEST-ALL is non-nil, test region without considering variables
 `cz-is-buffer-empty' and `cz-is-skip-start' (but `cz-is-test-size' is always
 considered).
 The recognition is absolutely magical and nothing should hard depend on it.
