@@ -1,16 +1,14 @@
-;; @(#) czech.el -- package for effective writing Czech things in Emacs
-
-;; @(#) $Id: czech.el,v 3.4 1997/02/26 20:32:07 pdm Exp $	
-;; @(#) $Keywords: i18n, Czech, keyboard $
-;; $KnownCompatibility: 19.34, XEmacs 19.14 $
-
-;; This file is *NOT* part of GNU Emacs nor XEmacs.
+;;; czech.el -- package for effective writing Czech things in Emacs
 
 ;; Copyright (C) 1995, 1996, 1997 Milan Zamazal
 
-;; Author:       Milan Zamazal <pdm@fi.muni.cz>
-;; Maintainer:   Milan Zamazal <pdm@fi.muni.cz>
-;; Remark:       Don't laugh too loudly while reading this file, please.
+;; Author:     Milan Zamazal <pdm@fi.muni.cz>
+;; Maintainer: Milan Zamazal <pdm@fi.muni.cz>
+;; Version:    $Id: czech.el,v 3.8 1997/11/08 10:20:29 pdm Exp $	
+;; Keywords:   i18n, Czech, keyboard
+;; KnownCompatibility: 19.34, XEmacs 19.14, 19.15
+
+;; This file is *NOT* part of GNU Emacs.
 
 ;; COPYRIGHT NOTICE
 ;;
@@ -62,7 +60,7 @@ It should be number between `0' and `9'.
 `0' means no messages except errors, `9' means print all messages.
 `5' or higher guarantees printing of all warnings.")
 
-(defvar cz-windoze (eq system-type 'windows-nt)
+(defconst cz-windoze (eq system-type 'windows-nt)
   "Indicates, whether it is Windoze NT version of Emacs.")
 
 (defun cz-message (level &rest message)
@@ -80,23 +78,27 @@ than value of the variable `cz-verbose-level'."
      8
      "Oh, I'm not sure if working properly under that hell system..."))
 
-(defvar cz-xemacs (string= (substring (emacs-version) 0 6) "XEmacs")
+(defconst cz-xemacs (string= (substring (emacs-version) 0 6) "XEmacs")
   "Indicates, whether we are running in XEmacs.")
 
 (defun cz-safe-eq (a b)
-  "If a's value is void, return nil and no error"
+  "If value of A is void, return nil and not error."
   (and (boundp a) (eq (symbol-value a) b)))
+
 (if cz-xemacs
     (cz-message
      8
      "Warning: Maybe Czech support does not work properly under XEmacs yet."))
 
-(defconst cz-version "$Id: czech.el,v 3.4 1997/02/26 20:32:07 pdm Exp $"
+(defconst cz-version "$Id: czech.el,v 3.8 1997/11/08 10:20:29 pdm Exp $"
   "Latest modification time and version number.")
 
 (defvar cz-load-hook nil
   "Hook run when package has been loaded.")
 
+(defvar cz-sk nil
+  "If non-nil, Slovak environment is set up.")
+
 (defvar cz-enable-keybindings t
   "Whether Czech functions should be assigned to global and/or local keys.")
 
@@ -139,10 +141,19 @@ MODE-STRING should start with space according to minor mode conventions."
 ;;; *** 8-bit characters ***
 
 (standard-display-european t)
+
 ;; It should be possible to use Czech support with external Czech keyboard
+
+(defvar cz-8bit-input t
+  "If nil, do not set 8-bit input.
+If you want to use emacs-czech with an external (not Emacsish) Czech keyboard,
+you must enable this.
+If you use emacs-czech with its own Czech keyboard, it is better to disable
+this, because it can cripple some keys in some situations (Linux console).")
+
 (set-input-mode (car (current-input-mode))
 		(nth 1 (current-input-mode))
-		0)
+		(if cz-8bit-input 0 nil))
 
 ;; Windoze set uses also 128-159 codes
 (if cz-windoze
@@ -235,31 +246,92 @@ and `cz-font-info-list'."
   t)
 
 
+;;; *** Calendar ***
+
+(defvar cz-calendar-inhibit nil
+  "If non-nil, Czech calendar style is not set.
+This variable takes effect only when loading `czech.el'.  Then there is no way
+to reset original calendar settings.
+On the other way, enabling Czech calendar style is possible any time by setting
+this variable to nil and reloading `czech.el'.")
+
+(defvar cz-calendar-all-holidays nil
+  "*If nil, only Czech (and not US) holidays will be shown.
+If variable `cz-calendar-inhibit is t, the value of `cz-calendar-all-holidays'
+does not matter.")
+
+(autoload 'cz-calendar-init "czech-calendar"
+  "Initialize Czech calendar settings and holidays.")
+
+(if (featurep 'calendar)
+    (cz-calendar-init))
+(add-hook 'calendar-load-hook 'cz-calendar-init)
+
+
+;;; *** Ispell ***
+
+(if (featurep 'czech)
+    ()
+  (setq ispell-dictionary-alist
+	(append (list (car ispell-dictionary-alist-1))
+		'(("czech"
+		   "[a-zA-Zةݮ]"
+		   "[^a-zA-Zةݮ]"
+		   "" nil ("-C" "-d" "czech") nil))
+		(cdr ispell-dictionary-alist-1)
+		ispell-dictionary-alist-2))
+  ;; We have to update menu too
+  ;; Has XEmacs "select dictionary" menu?
+  (if (not cz-xemacs)
+      (progn
+	(define-key ispell-menu-map [czech]
+	  (cons "Select Czech"
+		(list 'lambda () '(interactive)
+		      (list 'ispell-change-dictionary "czech"))))
+	(let ((czech-item (car (cdr ispell-menu-map)))
+	      (menu-map (cdr (cdr ispell-menu-map)))
+	      aux)
+	  (while (and (not (null (cdr menu-map)))
+		      (not (eq (car (car menu-map)) 'default)))
+	    (setq aux (cons (car menu-map) aux)
+		  menu-map (cdr menu-map)))
+	  (setq aux (cons (car menu-map) aux))
+	  (setcar menu-map czech-item)
+	  (setq ispell-menu-map (append '(keymap)
+					(nreverse aux)
+					menu-map))
+	  ;; Make changes visible
+	  (fset 'ispell-menu-map (symbol-value 'ispell-menu-map))))))
+
+
 ;;; *** Keyboard ***
 
 ;;; Variables involving using of Czech keyboard
 
+(defvar cz-keyboard-disable nil
+  "If non-nil, do not redefine keyboard in any way.")
+
 (defvar cz-global-mode nil
-  "If set non-`nil', Czech keyboard is global, else buffer local.
-Default setting is `nil', i.e. buffer local.
+  "If set non-nil, Czech keyboard is global, else buffer local.
+Default setting is nil, i.e. buffer local.
 You have to reload emacs-czech so that this change would take effect.")
 
 (defvar cz-keyboard-default-on nil
-  "If set non-`nil', Czech keyboard mode is switched on globally.
+  "If set non-nil, Czech keyboard mode is switched on globally.
 You have to reload emacs-czech so that this change would take effect.")
 
 (defvar cz-minibuffer-nonczech nil
-  "If non-`nil', minibuffer does not inherit cz-keyboard from calling buffer.
-If `nil', whenever you enter minibuffer, it is set to cz-keyboard-mode just if
+  "If non-nil, minibuffer does not inherit cz-keyboard from calling buffer.
+If nil, whenever you enter minibuffer, it is set to cz-keyboard-mode just if
 calling buffer is in cz-keyboard-mode.  If variable `cz-keyboard-default-on'
-or `cz-global-mode' is non-`nil', the value of `cz-minibuffer-nonczech' doesn't
+or `cz-global-mode' is non-nil, the value of `cz-minibuffer-nonczech' doesn't
 matter.
 You have to reload emacs-czech so that this change would take effect.")
 
 (defvar cz-keyboard-nonstandard t
-  "If set non-`nil', nonstandard Czech keyboard is used.
-If set just `t', default nonstandard keyboard is used.  If you want to use
-your own keyboard, set this variable to any value other than `nil' or `t'.
+  "If set non-nil, nonstandard Czech keyboard is used.
+If set just t, default nonstandard keyboard is used.  If you want to use
+your own keyboard, set this variable to any value other than nil or t.
 Standard Czech keyboard is probably defined by some official standard.
 As much I know it I defined its bindings here.
 Nonstandard Czech keyboard is defined by the following variables:
@@ -328,9 +400,53 @@ You have to reload emacs-czech so that this change would take effect.")
 ;; Standard Czech keyboard
 (defvar cz-standard-keyboard
   (if cz-windoze
+      (if cz-sk
+	  (progn
+	    (cz-message 1 "Sorry, Slovak keyboard not supported in Windoze.")
+	    nil)
+	'(
+	  ((?1 . "+") (?2 . [236]) (?3 . [154]) (?4 . [232]) (?5 . [248])
+	   (?6 . [158]) (?7 . [253]) (?8 . [225]) (?9 . [237]) (?0 . [233])
+	   (?` . "\"") (?~ . ";") (?! . "1") (?@ . "2") (?# . "3")
+	   (?$ . "4") (?% . "5") (?^ . "6") (?& . "7") (?* . "8")
+	   (?\( . "9") (?\) . "0") (?- . "=") (?_ . "%") (?\[ . [250])
+	   (?{ . "/") (?\] . ")") (?} . "(") (?\; . [249]) (?: . "\"")
+	   (?' . [167]) (?\" . "!") (?< . "?") (?> . ":") (?/ . "-")
+	   (?\? . "_") (?y . "z") (?Y . "Z") (?z . "y") (?Z . "Y"))
+	  (?+ (?c . [232]) (?d . [239]) (?e . [236]) (?n . [242]) (?r . [248])
+	      (?s . [154]) (?t . [157]) (?u . [249]) (?y . [190]) (?C . [200])
+	      (?D . [207]) (?E . [204]) (?N . [210]) (?R . [216]) (?S . [138])
+	      (?T . [141]) (?U . [217]) (?Y . [174])
+	      (?z . "z") (?Z . "Z"))
+	  (?= (?a . [225]) (?e . [233]) (?i . [237]) (?o . [243]) (?u . [250])
+	      (?z . [253]) (?A . [193]) (?E . [201]) (?I . [205]) (?O . [211])
+	      (?U . [218]) (?Z . [221])
+	      (?y . "y") (?Y . "Y"))))
+    (if cz-sk
+	;; Slovak keyboard
+	'(((?` . ";") (?~ . "^") (?1 . "+") (?! . "1") (?2 . [181]) (?@ . "2") 
+	   (?3 . [185]) (?# . "3") (?4 . [232]) (?$ . "4") (?5 . [187])
+	   (?% . "5") (?6 . [190]) (?^ . "6") (?7 . [253]) (?& . "7")
+	   (?8 . [225]) (?* . "8") (?9 . [237]) (?\( . "9") (?0 . [233])
+	   (?\) . "0") (?- . "=") (?_ . "%") (?\\ . [242]) (?| . "\)")
+	   (?Q . "`") (?\[ . [250]) (?{ . "/") (?\] . [228])
+	   (?} . "(") (?\; . [244]) (?: . "\"") (?' . [167]) (?\" . "!")
+	   (?< . "?") (?> . ":") (?/ . "-") (?\? . "_") (?y . "z") (?Y . "Z")
+	   (?z . "y") (?Z . "Y"))	  
+	  (?+ (?a . [228]) (?A . [196]) (?c . [232]) (?C . [200]) (?d . [239])
+	      (?D . [207]) (?e . [236]) (?E . [204]) (?l . [181]) (?L . [165])
+	      (?n . [242]) (?N . [210]) (?o . [244]) (?O . [212]) (?r . [248]) 
+	      (?R . [216]) (?s . [185]) (?S . [169]) (?t . [187]) (?T . [171])
+	      (?u . [249]) (?U . [217]) (?y . [190]) (?Y . [174]) (?z . "y") 
+	      (?Z . "Y"))
+	  (?= (?a . [225]) (?A . [193]) (?e . [233]) (?E . [201]) (?i . [237]) 
+	      (?I . [205]) (?l . [229]) (?L . [197]) (?o . [243]) (?O . [211]) 
+	      (?r . [224]) (?R . [192]) (?u . [250]) (?U . [218]) (?z . [253]) 
+	      (?Z . [221]) (?y . "z") (?Y . "Z")))
+      ;; Czech keyboard
       '(
-	((?1 . "+") (?2 . [236]) (?3 . [154]) (?4 . [232]) (?5 . [248])
-	 (?6 . [158]) (?7 . [253]) (?8 . [225]) (?9 . [237]) (?0 . [233])
+	((?1 . "+") (?2 . [236]) (?3 . [185]) (?4 . [232]) (?5 . [248])
+	 (?6 . [190]) (?7 . [253]) (?8 . [225]) (?9 . [237]) (?0 . [233])
 	 (?` . "\"") (?~ . ";") (?! . "1") (?@ . "2") (?# . "3")
 	 (?$ . "4") (?% . "5") (?^ . "6") (?& . "7") (?* . "8")
 	 (?\( . "9") (?\) . "0") (?- . "=") (?_ . "%") (?\[ . [250])
@@ -338,33 +454,15 @@ You have to reload emacs-czech so that this change would take effect.")
 	 (?' . [167]) (?\" . "!") (?< . "?") (?> . ":") (?/ . "-")
 	 (?\? . "_") (?y . "z") (?Y . "Z") (?z . "y") (?Z . "Y"))
 	(?+ (?c . [232]) (?d . [239]) (?e . [236]) (?n . [242]) (?r . [248])
-	    (?s . [154]) (?t . [157]) (?u . [249]) (?y . [190]) (?C . [200])
-	    (?D . [207]) (?E . [204]) (?N . [210]) (?R . [216]) (?S . [138])
-	    (?T . [141]) (?U . [217]) (?Y . [174])
+	    (?s . [185]) (?t . [187]) (?u . [249]) (?y . [190]) (?C . [200])
+	    (?D . [207]) (?E . [204]) (?N . [210]) (?R . [216]) (?S . [169])
+	    (?T . [171]) (?U . [217]) (?Y . [174])
 	    (?z . "z") (?Z . "Z"))
 	(?= (?a . [225]) (?e . [233]) (?i . [237]) (?o . [243]) (?u . [250])
 	    (?z . [253]) (?A . [193]) (?E . [201]) (?I . [205]) (?O . [211])
 	    (?U . [218]) (?Z . [221])
-	    (?y . "y") (?Y . "Y")))
-    '(
-      ((?1 . "+") (?2 . [236]) (?3 . [185]) (?4 . [232]) (?5 . [248])
-       (?6 . [190]) (?7 . [253]) (?8 . [225]) (?9 . [237]) (?0 . [233])
-       (?` . "\"") (?~ . ";") (?! . "1") (?@ . "2") (?# . "3")
-       (?$ . "4") (?% . "5") (?^ . "6") (?& . "7") (?* . "8")
-       (?\( . "9") (?\) . "0") (?- . "=") (?_ . "%") (?\[ . [250])
-       (?{ . "/") (?\] . ")") (?} . "(") (?\; . [249]) (?: . "\"")
-       (?' . [167]) (?\" . "!") (?< . "?") (?> . ":") (?/ . "-")
-       (?\? . "_") (?y . "z") (?Y . "Z") (?z . "y") (?Z . "Y"))
-      (?+ (?c . [232]) (?d . [239]) (?e . [236]) (?n . [242]) (?r . [248])
-	  (?s . [185]) (?t . [187]) (?u . [249]) (?y . [190]) (?C . [200])
-	  (?D . [207]) (?E . [204]) (?N . [210]) (?R . [216]) (?S . [169])
-	  (?T . [171]) (?U . [217]) (?Y . [174])
-	  (?z . "z") (?Z . "Z"))
-      (?= (?a . [225]) (?e . [233]) (?i . [237]) (?o . [243]) (?u . [250])
-	  (?z . [253]) (?A . [193]) (?E . [201]) (?I . [205]) (?O . [211])
-	  (?U . [218]) (?Z . [221])
-	  (?y . "y") (?Y . "Y"))))
-  "Definition list of standard Czech keyboard for `cz-set-keyboard'.")
+	    (?y . "y") (?Y . "Y")))))
+  "Definition list of standard Czech/Slovak keyboard for `cz-set-keyboard'.")
 
 (defvar cz-dead-symbols
   (if cz-windoze
@@ -377,11 +475,25 @@ Both DEAD-KEY and DISPLAY-CHARACTER must be characters.")
 
 ;; What about keypad keys?
 (defvar cz-redefine-keypad t
-  "If non-`nil', the package is allowed to redefine keypad keys.
+  "If non-nil, the package is allowed to redefine keypad keys.
 If you do not allow to do this, you may have problems to write numbers with
 Czech keyboard on the numeric block.
 You have to reload emacs-czech so that this change would take effect.")
 
+;; Chameleon keyboard
+(defvar cz-keyboard-test-function nil
+  "*Function called for determining proper keyboard.
+If non-nil, this function is called every time one of keys with different
+meanings on default and Czech keyboard is pressed.  This function should return
+non-nil just when the Czech keyboard should be used.
+
+This variable is intended to special purposes only.  Most time you will use
+setting of `cz-keyboard-mode' in mode hooks.
+
+You should be careful when setting this variable because of possible
+significant slow down of typing.")
+(make-variable-buffer-local 'cz-keyboard-test-function)
+
 
 ;;; *** ASCII display mode ***
 
@@ -470,20 +582,26 @@ contained in the list `cz-is-czech-characters'.")
 
 (defvar cz-is-czech-characters
   (if cz-windoze
+      (if cz-sk
+	  '(? ? ? ? ? ? ? ? ? ? ? ? ? ? ?
+	    ? ? ? ? ? ? ? ? ? ? ? ? ? ? ?)
+	'(? ? ? ? ? ? ? ? ? ? ? ? ? ? ?
+          ? ? ? ? ? ? ? ? ? ? ? ? ? ? ?))
+    (if cz-sk
+	'(? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ?
+	  ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ?)
       '(? ? ? ? ? ? ? ? ? ? ? ? ? ? ?
-	? ? ? ? ? ? ? ? ? ? ? ? ? ? ?)
-    '(? ? ? ? ? ? ? ? ? ? ? ? ? ? ?
-      ? ? ? ? ? ? ? ? ? ? ? ? ? ? ?))
+	? ? ? ? ? ? ? ? ? ? ? ? ? ? ?)))
   "List of characters to be considered beeing \"Czech\".")
 
 (autoload 'cz-is-cz-text "czech-misc"
   "Tries to recognize, whether current buffer contains ISO-8859-2 text.
 If the buffer is narrowed, only the narrowed part is consedered.
-If optional argument RETURN-RATIO is non-`nil', ratio (floating point number)
+If optional argument RETURN-RATIO is non-nil, ratio (floating point number)
 is returned instead of boolean value.  Ratio belongs to interval [-1;1] and
 equals to zero just when number of Czech and non-Czech characters from the
 above half of character set is the same.
-If TEST-ALL is non-`nil', test region without considering variables
+If TEST-ALL is non-nil, test region without considering variables
 `cz-is-buffer-empty' and `cz-is-skip-start' (but `cz-is-test-size' is always
 considered).
 The recognition is absolutely magical and nothing should hard depend on it.
@@ -519,18 +637,18 @@ specified in `cstools' package.")
 (defvar cz-convert-onebymore nil
   "*Whether to convert also the characters with more than one byte equivalent.
 This variable involves behaviour of the function `cz-convert-from-to'.
-If `nil', characters which should be replaced by more than one byte long string
+If nil, characters which should be replaced by more than one byte long string
 are replaced by `cz-convert-unknown-char'.
-If non-`nil', such characters are replaced by appropriate string.
+If non-nil, such characters are replaced by appropriate string.
 Opposite direction does not work: more than one-character strings are not
 replaced by their single character equivalent, they are replaced like any other
 characters.
-Warning: Setting this variable to non-`nil' can slow the conversion down.
+Warning: Setting this variable to non-nil can slow the conversion down.
 See also variable `cz-convert-unknown-char'.")
 
 (defvar cz-convert-unknown-char ?.
   "The character used as replacing for characters without one byte equivalent.
-If `nil', such characters remain untouched.
+If nil, such characters remain untouched.
 See variable `cz-convert-onebymore' for further explanation.
 Warning: If you change this variable you possibly should call
 `M-x cz-convert-delete-tables' so that created tables would be rebuilt.")
@@ -541,7 +659,7 @@ When the conversion by `cz-convert-from-to' function is asked, an appropriate
 conversion table must be created.  This make take some time.  If you want to
 eliminate most of this overhead, you can set this variable to some filename.
 Emacs will store newly created tables to this file for further usage.
-If the value of `cz-convert-cache-file' is `nil', no cache file will be used
+If the value of `cz-convert-cache-file' is nil, no cache file will be used
 and considered.")
 
 (defvar cz-convert-encfiles
@@ -562,7 +680,39 @@ and considered.")
   "*Minimal size of the converted region for verbose conversion.")
 
 (defvar cz-convert-auto-flag nil	; we need this for menu
-  "Non-`nil' just when autoconversion is enabled.")
+  "Non-nil just when autoconversion is enabled.")
+
+(defvar cz-convert-digraphs
+  '(("d" "dje")
+    ("t" "tje")
+    ("n" "nje")
+    ("b" "bje")
+    ("p" "pje")
+    ("v" "vje")
+    ("m" "mje")
+    (""  "aa")
+    (""  "ee")
+    (""  "ii")
+    (""  "oo")
+    (""  "uu")
+    (""  "uo" "uu")
+    (""  "yy")
+    (""  "cz" "cj")
+    (""  "rz" "rj")
+    (""  "sz" "sj")
+    (""  "dj")
+    (""  "nj")
+    (""  "tj")
+    (""  "zj" "zh" "zz"))
+  "Defines digraphs encoding.
+This is an association list with items like `(RIGHT-STRING . ENCHAINMENT)'.
+RIGHT-STRING can be string or character, ENCHAINMENT must be string.
+Enchainments are used by some people around Masaryk University and have no
+real importance.")
+
+(defvar cz-convert-digraphs-specchar ?|
+  "*Charachter inserted between ambigious digraph letters.
+The value is usually one of the characters `\\' or `|'")
 
 (autoload 'cz-convert-auto-encoding "czech-convert"
   "Sets encoding of the buffer for automatic encoding conversion.
@@ -583,10 +733,10 @@ off."
   t)
 (autoload 'cz-convert-auto-set "czech-convert"
     "Noninteractive variation of `cz-convert-auto-encoding'.
-If ENCODING is `nil', autoconversion is disabled for the buffer.")
+If ENCODING is nil, autoconversion is disabled for the buffer.")
 (autoload 'cz-convert-to-csech "czech-convert"
   "Converts region from ISO 8859-2 to 7-bit ASCII (so called \"csech\").
-If APPEND-MESSAGE is non-`nil' or prefix argument is given, it appends message
+If APPEND-MESSAGE is non-nil or prefix argument is given, it appends message
 about conversion to end of the converted region.  It converts all 8-bit
 characters, but those that are not diacritic will be converted to dots."
   t)
@@ -611,10 +761,20 @@ See also variables `cz-encoding-files-dir', `cz-convert-onebymore', and
 `cz-convert-cache-file'."
   t)
 (autoload 'cz-convert-undo$ "czech-convert"
-  "Removes those silly chars (^M and ^Z) from end of lines in buffer."
+  "Remove those silly chars (^M and ^Z) from end of lines in buffer."
   t)
 (autoload 'cz-convert-do$ "czech-convert"
-  "Adds that silly char (^M) to end of lines of the current buffer."
+  "Add that silly char (^M) to end of lines of the current buffer."
+  t)
+(autoload 'cz-convert-from-digraphs "czech-convert"
+  "Convert from \"sprezky\" encoding.
+That encoding is used by some people around Masaryk University and is of no
+importance."
+  t)
+(autoload 'cz-convert-to-digraphs "czech-convert"
+  "Convert to \"sprezky\" encoding.
+That encoding is used by some people around Masaryk University and is of no
+importance."
   t)
 
 
@@ -638,7 +798,7 @@ See documentation of `cz-sort' for more details."
 
 (autoload 'cz-sort-toggle "czech-sort"
   "Changes value of the variable `cz-sort'.
-Change is done by rotating values `nil', `t', and `by-mode'.
+Change is done by rotating values nil, t, and `by-mode'.
 If an optional prefix argument is given, rotation is done in opposite
 direction."
   t)
@@ -708,7 +868,7 @@ Follow regular expressions of respective newsgroups.
 See also variable `cz-message-method-news'.")
 
 (defvar cz-message-use-bbdb nil
-  "*If non-`nil', use BBDB for selecting message conversion method.
+  "*If non-nil, use BBDB for selecting message conversion method.
 You can use Insidious Big Brother Database for selecting message conversion
 method by specifying field `mailtype'.")
 
@@ -721,7 +881,7 @@ method by specifying field `mailtype'.")
 (autoload 'cz-message-show-method "czech-message"
   "Prints final encoding of the current message.
 
-If an optional argument QUIETLY is non-`nil', no message is printed, only
+If an optional argument QUIETLY is non-nil, no message is printed, only
 symbol value is returned."
   t)
 (autoload 'cz-message-send "czech-message"
@@ -733,6 +893,59 @@ symbol value is returned."
 (autoload 'cz-message-setup-mime "czech-message")
 
 
+;;; *** Adding tildes ***
+
+(defvar cz-tex-tilde-regexp-pair
+  '("\\([,:;(][ \t]+[ai]\\|\\<[AIKOSUVZkosuvz]\\)\\([ \t]+\\|[ \t]*\n[ \t]*\\)\\(\\w\\|\\\\\\)" . 2)
+  "*Regexp and number of matching place where tilde should be added.
+This is a pair (REGEXP . NUMBER), where REGEXP is regular expression matching
+the part of text which should contain missing tilde and NUMBER is the number of
+subexpression which should be replaced by tilde.
+The regexp is case sensitive.")
+
+(defvar cz-tex-tilde-string "~"
+  "*String inserted at places matching `cz-tex-tilde-regexp-pair'.
+For example it should be \"~\" for TeX and \"&nbsp;\" for SGML.
+You probably want to set it in major mode hooks.")
+(make-variable-buffer-local 'cz-tex-tilde-string)
+
+(defvar cz-tex-env-alist
+  '(("\\\\\\\\" . "")			; do not remove this
+    ("\\\\begin{verbatim}" . "\\\\end{verbatim}")
+    ("\\\\verb\\(.\\)" . (1))
+    ("\\$\\$" . "\\$\\$")
+    ("\\$" . "\\$")
+    ("\\\\(" . "\\\\)")
+    ("\\\\[[]" . "\\\\[]]")
+    ("\\\\begin{math}" . "\\\\end{math}")
+    ("\\\\begin{displaymath}" . "\\\\end{displaymath}")
+    ("\\\\begin{equation}" . "\\\\end{equation}")
+    ("\\\\begin{eqnarray\\*?}" . "\\\\end{eqnarray\\*?}"))
+  "*Association list of start-end pairs of regexps defining environments.
+Environments matching these strings are ignored during tildes addition
+process.
+Each list element is (START-EXPR . END-EXPR).
+START-EXPR is a regexp matching beginning of environment.
+END-EXPR is either a regexp matching end of environment or list of regexps and
+numbers which will be concatenated while replacing numbers with appropriate
+subexpressions of START-EXPR.")
+(make-variable-buffer-local 'cz-tex-env-alist)
+
+(autoload 'cz-tex-tildes-region "czech-tex"
+  "Add tildes in the given region.
+The tildes are added according to the following rules:
+- tildes are added after AIKOSUVZkosuvz
+- comments, maths, and verbatims are ignored
+- it is LaTeX oriented (sorry about it, but I do not know plain TeX)
+- no contingent line adjusting"
+  t)
+
+(autoload 'cz-tex-tildes "czech-tex"
+  "Add tildes in the current buffer.
+See `cz-tex-tildes-region for more info."
+  t)
+
+
 ;;; *** Saving settings ***
 
 (autoload 'cz-save-settings "czech-misc"
@@ -753,22 +966,17 @@ situations."
       ;; Console dependent part
       (if (string= (getenv "TERM") "linux")
 	  (progn
-	    (global-set-key "\e[P" 'cz-keyboard-mode)
-	    (global-set-key "\e\e[P" 'cz-correct-switch))
-	;; XEmacs dependent part
-	(if cz-xemacs
+	    (define-key function-key-map "\e[P" "\C-xzz")
+	    (define-key function-key-map "\e\e[P" "\C-xzx"))
+	(global-set-key [(meta pause)] 'cz-correct-switch)
+	(global-set-key [(meta break)] 'cz-correct-switch)
+	(global-set-key [(control pause)] 'cz-select-char)
+	(global-set-key [(control break)] 'cz-select-char)
+	(if (not cz-xemacs)
 	    (progn
-	      (global-set-key '(meta pause) 'cz-correct-switch)
-	      (global-set-key '(meta break) 'cz-correct-switch)
-	      (global-set-key '(control pause) 'cz-select-char)
-	      (global-set-key '(control break) 'cz-select-char))
-	  (global-set-key [?\e pause] 'cz-correct-switch)
-	  (global-set-key [?\e break] 'cz-correct-switch)
-	  (global-set-key [M-pause] 'cz-correct-switch)
-	  (global-set-key [M-break] 'cz-correct-switch)
-	  (global-set-key [C-pause] 'cz-select-char)
-	  (global-set-key [C-break] 'cz-select-char)))
-      ;; Common part
+	      (global-set-key [?\e pause] 'cz-correct-switch)
+	      (global-set-key [?\e break] 'cz-correct-switch))))
+      ;; Generic part
       (global-set-key [pause] 'cz-keyboard-mode)
       (global-set-key [break] 'cz-keyboard-mode)
       (global-set-key "\C-xz0" 'cz-keyboard-mode-off)
@@ -786,6 +994,7 @@ situations."
       (global-set-key "\C-xzt" 'cz-ascii-display-mode)
       (global-set-key "\C-xzu" 'cz-convert-undo$)
       (global-set-key "\C-xzw" 'cz-correct-word-switch)
+      (global-set-key "\C-xzx" 'cz-correct-switch)
       (global-set-key "\C-xzz" 'cz-keyboard-mode)))
 
 ;; Local maps
@@ -802,7 +1011,7 @@ situations."
 ;;; *** Menus ***
 
 (defvar cz-enable-menu t
-  "If non-`nil', define Czech menus.
+  "If non-nil, define Czech menus.
 You have to reload emacs-czech so that this change would take effect.")
 
 ;; We do this by defining very simple minor mode
@@ -837,37 +1046,57 @@ You have to reload emacs-czech so that this change would take effect.")
 (require 'easymenu)
 (let ((menu
        (list
-	"Czech" ':included 'cz-menu-mode
-	["Czech keyboard" cz-keyboard-mode :style toggle
-	 :selected cz-keyboard-mode]
+	(if cz-sk "Slovak" "Czech") ':included 'cz-menu-mode
+	(if cz-sk
+	    ["Slovak keyboard" cz-keyboard-mode :style toggle
+	     :selected cz-keyboard-mode]
+	  ["Czech keyboard" cz-keyboard-mode :style toggle
+	   :selected cz-keyboard-mode])
 	["Correct typed text" cz-correct-region t]
 	["Select character" cz-select-char t]
 	(list
 	 "Sorting"
 	 ["Always standard sorting" (setq cz-sort nil) :style radio
 	  :selected (not cz-sort)]
-	 ["Always Czech sorting" (progn (setq cz-sort t) (cz-sort-enable))
-	  :style radio :selected (eq cz-sort t)]
-	 ["Sorting by CZ keyboard mode"
-	  (progn (setq cz-sort 'by-mode) (cz-sort-enable)) :style radio
-	  :selected (eq cz-sort 'by-mode)])
+	 (if cz-sk
+	     ["Always Slovak sorting" (progn (setq cz-sort t) (cz-sort-enable))
+	      :style radio :selected (eq cz-sort t)]
+	   ["Always Czech sorting" (progn (setq cz-sort t) (cz-sort-enable))
+	    :style radio :selected (eq cz-sort t)])
+	 (if cz-sk
+	     ["Sorting by SK keyboard mode"
+	      (progn (setq cz-sort 'by-mode) (cz-sort-enable)) :style radio
+	      :selected (eq cz-sort 'by-mode)]
+	   ["Sorting by CZ keyboard mode"
+	    (progn (setq cz-sort 'by-mode) (cz-sort-enable)) :style radio
+	    :selected (eq cz-sort 'by-mode)]))
 	(append
 	 (if cz-xemacs
 	      (list "Message" :included (eq major-mode 'message-mode))
 	   (list "Message"))
 	 (list
-	  (list
-	   "Set method"
-	   ["MIME" (setq cz-message-this-method 'mime) :style radio
-	    :selected (cz-safe-eq 'cz-message-this-method 'mime)]
-	   ["plain" (setq cz-message-this-method 'plain) :style radio
-	    :selected (cz-safe-eq 'cz-message-this-method 'plain)]
-	   ["ASCII" (setq cz-message-this-method 'ascii) :style radio
-	    :selected (cz-safe-eq 'cz-message-this-method 'ascii)]
-	   "----"
-	   ["default" (setq cz-message-this-method 'default) :style radio
-	    :selected (or (cz-safe-eq 'cz-message-this-method 'default)
-			  (cz-safe-eq 'cz-message-this-method nil))]))
+	  (if cz-xemacs
+	      (list
+	       "Set method"
+	       ["MIME" (setq cz-message-this-method 'mime) :style radio
+		:included (eq cz-gnus t)
+		:selected (cz-safe-eq 'cz-message-this-method 'mime)]
+	       ["plain" (setq cz-message-this-method 'plain) :style radio
+		:selected (cz-safe-eq 'cz-message-this-method 'plain)]
+	       ["ASCII" (setq cz-message-this-method 'ascii) :style radio
+		:selected (cz-safe-eq 'cz-message-this-method 'ascii)]
+	       "----"
+	       ["default" (setq cz-message-this-method 'default) :style radio
+		:selected (or (cz-safe-eq 'cz-message-this-method 'default)
+			      (cz-safe-eq 'cz-message-this-method nil))])
+	    (list
+	     "Set method"
+	     ["MIME" (setq cz-message-this-method 'mime) :style radio
+	      :selected (not (eq cz-gnus t))]
+	     ["plain" (setq cz-message-this-method 'plain) :style radio]
+	     ["ASCII" (setq cz-message-this-method 'ascii) :style radio]
+	     "----"
+	     ["default" (setq cz-message-this-method 'default) :style radio])))
 	 (list
 	  ["Show method" cz-message-show-method t]
 	  ["Send and exit..." cz-message-send-and-exit t]))
@@ -889,7 +1118,6 @@ You have to reload emacs-czech so that this change would take effect.")
 	(if cz-windoze
 	    ["Convert to Windoze" cz-convert-to-native t]
 	  ["Convert to ISO 8859-2" cz-convert-to-native t])
-	["Convert to ISO 8859-2" cz-convert-to-native t]
 	["Convert to ASCII" cz-convert-to-csech t]
 	["Convert from to..." cz-convert-from-to t]
 	["ASCII display" cz-ascii-display-mode :style toggle
@@ -967,10 +1195,12 @@ Key bindings are set automatically now."
     (cz-set-fonts cz-font-default cz-font-faces-list cz-font-info-list))
 
 ;; Keyboard
-(require 'czech-keyboard)		; we redefine keyboard always
-(cz-set-keyboard nil (not cz-redefine-keypad))
-(if (not cz-xemacs)
-    (define-key isearch-mode-map [pause] 'cz-handle-key-isearch))
+(if (not cz-keyboard-disable)
+    (progn
+      (require 'czech-keyboard)
+      (cz-set-keyboard nil (not cz-redefine-keypad))
+      (if (not cz-xemacs)
+	  (define-key isearch-mode-map [pause] 'cz-handle-key-isearch))))
 
 ;; Sorting
 (if cz-sort
